package de.tud.kom.p2psim.impl.topology.movement.vehicular.sumo.simulation.controller.xml;

import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import de.tud.kom.p2psim.impl.topology.movement.vehicular.information.Position;
import de.tud.kom.p2psim.impl.topology.movement.vehicular.sumo.simulation.controller.SimulationSetupExtractor;
import de.tud.kom.p2psim.impl.topology.movement.vehicular.sumo.simulation.controller.VehicleController;
import de.tud.kom.p2psim.impl.topology.movement.vehicular.sumo.simulation.controller.csv.RoadSideUnitInformationHandler;

public class XMLSimulationController implements VehicleController, SimulationSetupExtractor, Runnable {
    private String _vehicleDataPath;
    private String _roadSideUnitDataPath;

    private List<String> _vehicles;

    private Map<Double, Map<String, Position>> _positonsByTimestamp = new HashMap<>();
    private int _futureInformation = 10;

    private boolean _initalized = false;

    private VehicleDataInformationHandler _vehicleDataInformationHandler = new VehicleDataInformationHandler();
    private RoadSideUnitInformationHandler _roadSideUnitInformationHandler = new RoadSideUnitInformationHandler();

    private double _start = -1;

    private double _step;

    public XMLSimulationController(String pVehicleDataPath, String pRoadSideUnitDataPath) {
        this(pVehicleDataPath);
        _roadSideUnitDataPath = pRoadSideUnitDataPath;
    }

    public XMLSimulationController(String pVehicleDataPath) {
        _vehicleDataPath = pVehicleDataPath;
    }

    @Override
    public void init() {

        if (!_initalized) {
            if (_vehicleDataPath != null) {
                Thread thread = new Thread(this);
                thread.start();

                for (int i = 0; i < _futureInformation; i++) {
                    nextStep();
                }
            }
            if (_roadSideUnitDataPath != null) {
                _roadSideUnitInformationHandler.parseIntersectionsFile(_roadSideUnitDataPath);
            }
            _initalized = true;
        }
    }

    @Override
    public Position getVehiclePosition(String pVehicleID) {
        return getVehiclePosition(_step, pVehicleID);
    }

    public Position requestVehiclePosition(String pVehicleID) {
        return _vehicleDataInformationHandler.getVehiclePositions().get(pVehicleID);
    }

    @Override
    public boolean nextStep() {
        _vehicleDataInformationHandler.readNext();

        _step = _vehicleDataInformationHandler.getStep() - _futureInformation;
        if (_start == -1) {
            _start = _vehicleDataInformationHandler.getStep();
        }

        double newStep = _step + _futureInformation;
        if (!_positonsByTimestamp.containsKey(newStep)) {
            Map<String, Position> vehiclePositions = new HashMap<>();
            _positonsByTimestamp.put(newStep, vehiclePositions);

            List<String> allVehicles = new ArrayList<>(_vehicleDataInformationHandler.getVehiclePositions().keySet());
            for (String vehicle : allVehicles) {
                Position vehiclePosition = requestVehiclePosition(vehicle);
                if (vehiclePosition != null) {
                    vehiclePositions.put(vehicle, vehiclePosition);
                }
            }
        }

        return !_vehicleDataInformationHandler.isTerminated();
    }

    @Override
    public List<String> getAllVehicles() {
        return getAllVehicles(_step);
    }

    @Override
    public double getStep() {
        return _step;
    }

    @Override
    public double getStart() {
        return _start;
    }

    @Override
    public void run() {
        try {
            SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
            parser.parse(new FileInputStream(_vehicleDataPath), _vehicleDataInformationHandler);
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void setObservedArea(double pStartX, double pStartY, double pEndX, double pEndY) {
        _vehicleDataInformationHandler.setObservedArea(pStartX, pStartY, pEndX, pEndY);
        _roadSideUnitInformationHandler.setObservedArea(pStartX, pStartY, pEndX, pEndY);
    }

    @Override
    public Position getVehiclePosition(double pStep, String pVehicleID) {
        Map<String, Position> map = _positonsByTimestamp.get(pStep);

        return map.get(pVehicleID);
    }

    @Override
    public List<String> getAllVehicles(double pStep) {
        Map<String, Position> map = _positonsByTimestamp.get(pStep);

        return new ArrayList<>(map.keySet());
    }

    @Override
    public double getMaximumAvailablePrediction() {
        double max = Collections.max(_positonsByTimestamp.keySet());

        return max;
    }

    @Override
    public List<Position> getAllIntersections(boolean pCluster) {
        return _roadSideUnitInformationHandler.getIntersections();
    }

}
